<?php

namespace App\Http\Controllers\API_MOBILE\Entreprise;

use App\Http\Controllers\Controller;
use App\Http\Resources\EntrepriseResource;
use App\Models\Entreprise;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class EntrepriseController_MOBILE extends Controller
{
    public function index()
    {
        try {
            $entreprise = Entreprise::paginate(20);
            return  EntrepriseResource::collection($entreprise);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }



    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "ICE" => "required|integer|unique:entreprises",
                "Email" => "required|email|unique:entreprises,email|max:255",
                "Telephone" => "required|string|max:20",
                "Adresse" => "required|string|max:255",
                "Pays" => "required|string|max:255",
                "Ville" => "required|string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $entreprise = new Entreprise();
            $entreprise->Name = $request->input('Name');
            $entreprise->Email = $request->input('Email');
            $entreprise->Telephone = $request->input('Telephone');
            $entreprise->ICE = $request->input('ICE');
            $entreprise->Adresse = $request->input('Adresse');
            $entreprise->Pays = $request->input('Pays');
            $entreprise->Ville = $request->input('Ville');

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;


                Storage::disk('public')->put('EntrepriseImages/' . $imageName, $imageData);


                $entreprise->Image = 'EntrepriseImages/' . $imageName;


                $entreprise
                ->addMedia(storage_path('app/public/EntrepriseImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("EntrepriseImages" , "EntrepriseImages");
            }

            $entreprise->save();

            DB::commit();

            return response()->json(["message" => "Enterprise created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {


            $entreprise = Entreprise::findOrFail($id)->paginate(20);
            return EntrepriseResource::collection($entreprise);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Enterprise not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {

        try {

            $entreprise = Entreprise::findOrFail($id);

            $validator = Validator::make($request->all(),[
                "Name" => "string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "ICE" => "integer|unique:entreprises",
                "Email" => "email|unique:entreprises,email|max:255",
                "Telephone" => "string|max:20",
                "Adresse" => "string|max:255",
                "Pays" => "string|max:255",
                "Ville" => "string|max:255"

            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $entreprise->fill($request->all());

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('EntrepriseImages/' . $imageName, $imageData);


                $entreprise->Image = 'EntrepriseImages/' . $imageName;

                // Delete the existing image from the media library
                if ($entreprise->getFirstMedia("EntrepriseImages")) {
                    $entreprise->getFirstMedia("EntrepriseImages")->forceDelete();
                }

                $entreprise
                ->addMedia(storage_path('app/public/EntrepriseImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("EntrepriseImages" , "EntrepriseImages");
            }

            $entreprise->save();

            DB::commit();

            return response()->json(["message" => "Enterprise updated successfully ", "status" => 200]);

        }  catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {

            $entreprise = Entreprise::findOrFail($id);

            DB::beginTransaction();

            // Delete the image associated with the enterprise from the media library
            if ($entreprise->getFirstMedia("EntrepriseImages")) {
                $entreprise->getFirstMedia("EntrepriseImages")->forceDelete();
            }

            // Delete the enterprise
            $entreprise->forceDelete();

            DB::commit();

            return response()->json(["message" => "Enterprise deleted successfully!", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Enterprise not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
