<?php

namespace App\Http\Controllers\API_MOBILE\Permission;

use App\Http\Controllers\Controller;
use App\Http\Resources\PermissionResource;
use App\Models\Entreprise;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Spatie\Permission\Models\Permission;

class PermissionController_MOBILE extends Controller
{
    public function index()
    {
        try {
            $permission = Permission::paginate(20);
            return PermissionResource::collection($permission);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where("ICE", $request->input("EntrepriseICE"))->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found");
            }

            DB::beginTransaction();

            // Create the role with the correct guard
            $Permission = Permission::create([
                "name" => $request->input("Name"),
                "guard_name" => 'api' // Ensure the guard name matches the one used in your application
            ]);

            // Assuming Entreprise model uses the 'API' guard
            $entreprise->givePermissionTo($Permission);

            DB::commit();

            return response()->json(["message" => "Permission created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {

        try {
            // Find the enterprise by id
            $entreprise = Entreprise::where("id", $id)->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found!!");
            }

            // Fetch all permissions associated with the enterprise
            $model_has_permissions = DB::table("model_has_permissions")
                ->where("model_id", $entreprise->id)
                ->where("model_type", Entreprise::class) // Ensure you filter by the correct model type
                ->get();

            // Retrieve permission details using the permission IDs
            $permissions = Permission::whereIn('id', $model_has_permissions->pluck('permission_id'))->paginate(20);
            return PermissionResource::collection($permissions);

        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            // Validate the incoming request
            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            // Find the enterprise by ICE
            $entreprise = Entreprise::where("ICE", $request->input("EntrepriseICE"))->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found");
            }

            DB::beginTransaction();

            // Find the permission association by ID
            $model_has_permission = DB::table('model_has_permissions')->where([
                "permission_id" => $id,
                "model_id" => $entreprise->id,
                "model_type" => Entreprise::class // Ensure the correct model type is used
            ])->first();

            if (!$model_has_permission) {
                throw new ModelNotFoundException("Model has Permission Not Found!!");
            }

            // Find the permission by ID
            $permission = Permission::where("id", $model_has_permission->permission_id)->first();
            if (!$permission) {
                throw new ModelNotFoundException("Permission Not Found!!");
            }

            // Update the permission attributes
            $permission->update([
                "name" => $request->input("Name"),
                "guard_name" => 'api' // Ensure the guard name matches the one used in your application
            ]);

            DB::commit();

            return response()->json(["message" => "Permission updated successfully!", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request,string $id)
    {

        try {


            $validator = Validator::make($request->all(), [
                "PermissionId"=>"required|integer|exists:permissions,id"
         ]);

         if ($validator->fails()) {
            throw ValidationException::withMessages($validator->errors()->toArray());
        }


            DB::beginTransaction();

            $entreprise = Entreprise::where("id", $id)->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found!!");
            }


            $model_has_permission = DB::table("model_has_permissions")
                ->where(["model_id"=> $entreprise->id , "permission_id"=>$request->input("PermissionId")])->first();

            if ($model_has_permission) {
                // Find the role using the role_id
                $permission = Permission::where("id", $model_has_permission->permission_id)->first();

                if ($permission) {
                    // Remove the  association
                    $entreprise->revokePermissionTo($permission);
                    $permission->forceDelete();
                }
            }
            DB::commit();

            return response()->json(["message" => "Permission deleted successfully!", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
