<?php

namespace App\Http\Controllers\API_MOBILE\Role;

use App\Http\Controllers\Controller;
use App\Http\Resources\RoleResource;
use App\Models\Entreprise;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Spatie\Permission\Models\Role;

class RoleController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $role = Role::paginate(20);
            return RoleResource::collection($role);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where("ICE", $request->input("EntrepriseICE"))->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found");
            }

            DB::beginTransaction();

            // Create the role with the correct guard
            $role = Role::create([
                "name" => $request->input("Name"),
                "guard_name" => 'api' // Ensure the guard name matches the one used in your application
            ]);

            // Assuming Entreprise model uses the 'web' guard
            $entreprise->assignRole($role);

            DB::commit();

            return response()->json(["message" => "Role created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {

        try {


          ;
            // Find the enterprise by ICE
            $entreprise = Entreprise::where("id", $id)->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found!!");
            }

            // Fetch the role associated with the enterprise
            $model_has_role = DB::table("model_has_roles")
                ->where("model_id", $entreprise->id)
                ->where("model_type", Entreprise::class) // Ensure you filter by the correct model type
                ->first();

            if (!$model_has_role) {
                throw new ModelNotFoundException("Role association not found for the given enterprise.");
            }

            // Retrieve the role details
            $role = Role::where("id", $model_has_role->role_id)->paginate(20);

            if (!$role) {
                throw new ModelNotFoundException("Role Not Found!!");
            }

            return RoleResource::collection($role);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {


            // Validate the incoming request
            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            // Find the enterprise by ICE
            $entreprise = Entreprise::where("ICE", $request->input("EntrepriseICE"))->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found");
            }

            DB::beginTransaction();

            // Find the role association by ID
            $model_has_role = DB::table('model_has_roles')->where([
                "role_id" => $id,
                "model_id" => $entreprise->id,
                "model_type" => Entreprise::class // Ensure the correct model type is used
            ])->first();

            if (!$model_has_role) {
                throw new ModelNotFoundException("Model has Role Not Found!!");
            }

            // Find the role by ID
            $role = Role::find($model_has_role->role_id);
            if (!$role) {
                throw new ModelNotFoundException("Role Not Found!!");
            }

            // Update the role attributes
            $role->update([
                "name" => $request->input("Name"),
                "guard_name" => 'api' // Ensure the guard name matches the one used in your application
            ]);

            DB::commit();

            return response()->json(["message" => "Role updated successfully!", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request,string $id)
    {

        try {


            $validator = Validator::make($request->all(), [
                "RoleId"=>"required|integer|exists:roles,id"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }


            DB::beginTransaction();


            $entreprise = Entreprise::where("ICE", $id)->first();
            if (!$entreprise) {
                throw new ModelNotFoundException("Enterprise Not Found!!");
            }


            $model_has_role = DB::table("model_has_roles")
                ->where(["model_id"=> $entreprise->id , "role_id"=> $request->input("RoleId")])
                ->first();

            if ($model_has_role) {
                // Find the role using the role_id
                $role = Role::where("id", $model_has_role->role_id)->first();


                if ($role) {
                    // Remove the role association
                    $entreprise->removeRole($role);
                    $role->forceDelete();
                }
            }
            DB::commit();

            return response()->json(["message" => "Role deleted successfully!", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
