<?php

namespace App\Http\Controllers\API_MOBILE\User;

use App\Http\Controllers\Controller;
use App\Http\Resources\UserResource;
use App\Models\Entreprise;
use App\Models\User;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;

class UserController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $User = User::paginate(20);
            return UserResource::collection($User);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "EntrepriseICE" =>"required|integer|exists:entreprises,ICE",
                "Email" => "required|email|unique:users,email|max:255",
                "Password" => "required|string|min:8",
                "Telephone" => "required|string|max:20",
                "Adresse" => "required|string|max:255",
                "Ville" => "required|string|max:255",
                "roleId"=>"required|integer|exists:roles,id",
                "permissions" => "sometimes|array",
                "permissions.*" => "integer|exists:permissions,id"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }

            DB::beginTransaction();

            $User = new User();
            $User->Name = $request->input('Name');
            $User->EntrepriseId = $entreprise->id;
            $User->Email = $request->input('Email');
            $User->Telephone = $request->input('Telephone');
            $User->Password = bcrypt($request->input('Password'));
            $User->Adresse = $request->input('Adresse');
            $User->Ville = $request->input('Ville');

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;


                Storage::disk('public')->put('UserImages/' . $imageName, $imageData);


                $User->Image = 'UserImages/' . $imageName;


                $User
                ->addMedia(storage_path('app/public/UserImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("UserImages" , "UserImages");
            }

            $User->save();



            //Role And Permissions Managment

            $Role = Role::findOrFail($request->input("roleId"));
            $User->assignRole($Role->id);

            if ($request->has('permissions')) {
                $permissions = Permission::whereIn('id', $request->input('permissions'))->get();
                $User->givePermissionTo($permissions);
            }

            DB::commit();

            return response()->json(["message" => "User created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {

            $user = User::where("EntrepriseId",$id)->paginate(20);
            return UserResource::collection($user);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "User not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }


    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {



            $user = User::findOrFail($id);

            $validator = Validator::make($request->all(),[
                "Name" => "string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "Email" => "email|unique:users,email|max:255",
                "Telephone" => "string|max:20",
                "Password" => "required|string|min:8",
                "Adresse" => "string|max:255",
                "Pays" => "string|max:255",
                "Ville" => "string|max:255",
                "roleId" => "sometimes|integer|exists:roles,id",
                "permissions" => "sometimes|array",
                "permissions.*" => "integer|exists:permissions,id"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $user->fill($request->all());

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('UserImages/' . $imageName, $imageData);


                $user->Image = 'UserImages/' . $imageName;

                // Delete the existing image from the media library
                if ($user->getFirstMedia("UserImages")) {
                    $user->getFirstMedia("UserImages")->forceDelete();
                }

                $user
                ->addMedia(storage_path('app/public/UserImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("UserImages" , "UserImages");
            }

            $user->save();

              // Update roles if provided
        if ($request->has('roleId')) {
            $role = Role::findOrFail($request->input('roleId'));
            $user->syncRoles([$role]);
        }

        // Update permissions if provided
        if ($request->has('permissions')) {
            $permissions = Permission::whereIn('id', $request->input('permissions'))->get();
            $user->syncPermissions($permissions);
        }

            DB::commit();

            return response()->json(["message" => "User updated successfully ", "status" => 200]);

        }  catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {


            $user = User::findOrFail($id);
            DB::beginTransaction();

            $user->forceDelete();

            DB::commit();

            return response()->json(["message" => "User deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "User not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
